using System;
using UnityEngine;

namespace Stopper {
	public class SnapControllerToSurface : MVRScript {
		
		int collisionLayer = 1;
		
		GameObject controllerMeshContainer;
		FreeControllerV3 sampleController;
		MeshRenderer meshRenderer;
		MeshFilter meshFilter;
		Mesh controllerMesh;
		
		JSONStorableFloat handOffset;
		
		public override void Init() {
			controllerMeshContainer = new GameObject("SnapControllerToSurface");
			controllerMeshContainer.transform.parent = containingAtom.transform;
			controllerMeshContainer.SetActive(true);
			
			meshRenderer = controllerMeshContainer.AddComponent<MeshRenderer>();
			meshFilter = controllerMeshContainer.AddComponent<MeshFilter>();
			
			collisionLayer |= 1; // Base
			collisionLayer |= 1<<12; // Tongue
			collisionLayer |= 1<<14; // PhysicsMesh1
			collisionLayer |= 1<<15; // PhysicsMesh2
			collisionLayer |= 1<<26; // Character
			collisionLayer |= 1<<29; // CharacterExtremities
			
			// collisionLayer |= 1<<28; // Hair

			CreateTextField(new JSONStorableString("HandOffset", "Moves the snap point of the hands towards the fingertips. Palm is at approximately 0.07, fingertips at 0.16 "), false).height = 10;
			handOffset = new JSONStorableFloat("Hand Offset", 0.07f, 0f, 0.3f, true, true);
			RegisterFloat(handOffset);
			var handOffsetSlider = CreateSlider(handOffset, false);
			handOffsetSlider.valueFormat = "F3";
		}
		
		private void Update(){
			FreeControllerV3 selectedController = SuperController.singleton.GetSelectedController();
			if ((Input.GetKey(KeyCode.LeftControl) || Input.GetKey(KeyCode.LeftAlt)) && selectedController ) {
				
				if(controllerMesh == null){
					meshRenderer.material = selectedController.material;
					controllerMesh = selectedController.moveModeOverlayMesh;
					meshFilter.mesh = controllerMesh;
				}
				
				bool updateRotation = Input.GetKey(KeyCode.LeftAlt);
				meshRenderer.enabled = true;
				
				RaycastHit hit;
				Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
				if (Physics.Raycast(ray, out hit, 100f,collisionLayer)) {
					Vector3 point = hit.point;
					controllerMeshContainer.transform.position = hit.point;
					
					if(updateRotation){
						controllerMeshContainer.transform.rotation = Quaternion.FromToRotation(selectedController.transform.up, hit.normal) * selectedController.transform.rotation;
					} else {
						controllerMeshContainer.transform.rotation = selectedController.transform.rotation;
					}
				}
				
				// RightClick
				if (Input.GetMouseButtonDown(1)){
					selectedController.transform.rotation = controllerMeshContainer.transform.rotation;

					if(selectedController.name.ToLower().Contains("handcontrol")){
						Vector3 offset =  controllerMeshContainer.transform.right * handOffset.val;
						selectedController.transform.position = controllerMeshContainer.transform.position + offset;
					} else {
						selectedController.transform.position = controllerMeshContainer.transform.position;
					}
				}
			} else {
				meshRenderer.enabled = false;
			}
		}
		
		private void OnDisable()
		{
		}
		
		private void OnDestroy(){
			if(controllerMeshContainer != null){
				Destroy(controllerMeshContainer);
			}
		}
	}
}
